Qt C++ Code Review
x-tools-author/x-tools
Read-only review of Qt6 C++ code that combines a deterministic lint script with six parallel analysis agents and reports only high-confidence issues.
Map Intel GPU ISA instruction addresses to precise SYCL/DPC++ source file:line numbers.
$ npx skills add intel/torch-xpu-ops --skill asm-source-mapping -a claude-codeProject install by default; add -g for ~/.claude/skills/.
$ gh skill install intel/torch-xpu-ops asm-source-mapping --agent claude-codeProject scope by default; add --scope user for a personal install. Needs GitHub CLI 2.90.0 or later (public preview).
$ git clone --depth 1 https://github.com/intel/torch-xpu-ops.git skills-src && mkdir -p .claude/skills && cp -r skills-src/.claude/skills/asm-source-mapping .claude/skills/asm-source-mapping && rm -rf skills-srcUse ~/.claude/skills/ instead of .claude/skills for a personal install. The folder must contain SKILL.md.
Claude Code skills documentation · loads skills from .claude/skills/
Install the "asm-source-mapping" agent skill from https://github.com/intel/torch-xpu-ops/tree/main/.claude/skills/asm-source-mapping into .claude/skills/asm-source-mapping/ in this project. Copy the whole folder (SKILL.md and every file beside it), keep the folder name "asm-source-mapping", then confirm the skill loads.Claude Code copies the folder itself, the same result as the manual copy. Check what it changed before you commit it.
$skill-installer install https://github.com/intel/torch-xpu-ops/tree/main/.claude/skills/asm-source-mappingType this inside Codex. $skill-installer <name> installs a curated skill from openai/skills. The installer writes to $CODEX_HOME/skills (default ~/.codex/skills). Restart Codex if the skill does not show up.
$ npx skills add intel/torch-xpu-ops --skill asm-source-mapping -a codexProject install goes to .agents/skills/; add -g for ~/.codex/skills/.
$ gh skill install intel/torch-xpu-ops asm-source-mapping --agent codexProject scope by default (.agents/skills/); add --scope user for a personal install.
$ git clone --depth 1 https://github.com/intel/torch-xpu-ops.git skills-src && mkdir -p .agents/skills && cp -r skills-src/.claude/skills/asm-source-mapping .agents/skills/asm-source-mapping && rm -rf skills-srcUse ~/.agents/skills/ instead of .agents/skills for a personal install.
Codex skills documentation · loads skills from .agents/skills/
Install the "asm-source-mapping" agent skill from https://github.com/intel/torch-xpu-ops/tree/main/.claude/skills/asm-source-mapping into .agents/skills/asm-source-mapping/ in this project. Copy the whole folder (SKILL.md and every file beside it), keep the folder name "asm-source-mapping", then confirm the skill loads.Codex copies the folder itself, the same result as the manual copy. Check what it changed before you commit it.
$ npx skills add intel/torch-xpu-ops --skill asm-source-mapping -a cursorProject install goes to .agents/skills/; add -g for ~/.cursor/skills/.
$ gh skill install intel/torch-xpu-ops asm-source-mapping --agent cursorProject scope by default (.agents/skills/); add --scope user for a personal install.
$ git clone --depth 1 https://github.com/intel/torch-xpu-ops.git skills-src && mkdir -p .cursor/skills && cp -r skills-src/.claude/skills/asm-source-mapping .cursor/skills/asm-source-mapping && rm -rf skills-srcUse ~/.cursor/skills/ instead of .cursor/skills for a personal install.
Cursor skills documentation · loads skills from .cursor/skills/, .agents/skills/, .claude/skills/, .codex/skills/
Install the "asm-source-mapping" agent skill from https://github.com/intel/torch-xpu-ops/tree/main/.claude/skills/asm-source-mapping into .cursor/skills/asm-source-mapping/ in this project. Copy the whole folder (SKILL.md and every file beside it), keep the folder name "asm-source-mapping", then confirm the skill loads.Cursor copies the folder itself, the same result as the manual copy. Check what it changed before you commit it.
$ gemini skills install https://github.com/intel/torch-xpu-ops.git --path .claude/skills/asm-source-mapping--scope user (default) or --scope workspace; --path is the subfolder of the repo that holds the skill; --consent skips the security confirmation prompt.
$ npx skills add intel/torch-xpu-ops --skill asm-source-mapping -a gemini-cliProject install goes to .agents/skills/; add -g for ~/.gemini/skills/.
$ gh skill install intel/torch-xpu-ops asm-source-mapping --agent gemini-cliProject scope by default (.agents/skills/); add --scope user for a personal install.
$ git clone --depth 1 https://github.com/intel/torch-xpu-ops.git skills-src && mkdir -p .gemini/skills && cp -r skills-src/.claude/skills/asm-source-mapping .gemini/skills/asm-source-mapping && rm -rf skills-srcUse ~/.gemini/skills/ instead of .gemini/skills for a personal install, then run /skills reload.
Gemini CLI skills documentation · loads skills from .gemini/skills/, .agents/skills/
Install the "asm-source-mapping" agent skill from https://github.com/intel/torch-xpu-ops/tree/main/.claude/skills/asm-source-mapping into .gemini/skills/asm-source-mapping/ in this project. Copy the whole folder (SKILL.md and every file beside it), keep the folder name "asm-source-mapping", then confirm the skill loads.Gemini CLI copies the folder itself, the same result as the manual copy. Check what it changed before you commit it.
$ gh skill install intel/torch-xpu-ops asm-source-mappingInstalls for Copilot at project scope by default; add --scope user for a personal install. Preview a skill first with gh skill preview. Needs GitHub CLI 2.90.0 or later (public preview).
$ npx skills add intel/torch-xpu-ops --skill asm-source-mapping -a github-copilotProject install goes to .agents/skills/; add -g for ~/.copilot/skills/.
$ git clone --depth 1 https://github.com/intel/torch-xpu-ops.git skills-src && mkdir -p .github/skills && cp -r skills-src/.claude/skills/asm-source-mapping .github/skills/asm-source-mapping && rm -rf skills-srcUse ~/.copilot/skills/ instead of .github/skills for a personal install. Commit .github/skills so cloud agent and code review can use it.
GitHub Copilot skills documentation · loads skills from .github/skills/, .claude/skills/, .agents/skills/
Install the "asm-source-mapping" agent skill from https://github.com/intel/torch-xpu-ops/tree/main/.claude/skills/asm-source-mapping into .github/skills/asm-source-mapping/ in this project. Copy the whole folder (SKILL.md and every file beside it), keep the folder name "asm-source-mapping", then confirm the skill loads.GitHub Copilot copies the folder itself, the same result as the manual copy. Check what it changed before you commit it.
$ npx skills add intel/torch-xpu-ops --skill asm-source-mapping -a opencodeOpenCode documents no install command of its own. Project install goes to .agents/skills/; add -g for ~/.config/opencode/skills/.
$ gh skill install intel/torch-xpu-ops asm-source-mapping --agent opencodeProject scope by default (.agents/skills/); add --scope user for a personal install.
$ git clone --depth 1 https://github.com/intel/torch-xpu-ops.git skills-src && mkdir -p .opencode/skills && cp -r skills-src/.claude/skills/asm-source-mapping .opencode/skills/asm-source-mapping && rm -rf skills-srcUse ~/.config/opencode/skills/ instead of .opencode/skills for a personal install.
OpenCode skills documentation · loads skills from .opencode/skills/, .claude/skills/, .agents/skills/
Install the "asm-source-mapping" agent skill from https://github.com/intel/torch-xpu-ops/tree/main/.claude/skills/asm-source-mapping into .opencode/skills/asm-source-mapping/ in this project. Copy the whole folder (SKILL.md and every file beside it), keep the folder name "asm-source-mapping", then confirm the skill loads.OpenCode copies the folder itself, the same result as the manual copy. Check what it changed before you commit it.
asm-source-mappingMap Intel GPU ISA instruction addresses to precise SYCL/DPC++ source file:line numbers.
Asm Source Mapping is an agent skill from intel/torch-xpu-ops, published by the product's own GitHub organization. Map Intel GPU ISA instruction addresses to precise SYCL/DPC++ source file:line numbers. Primary method reads the DWARF .debugline section from the GPU zebin ELF. Fallback uses structural pattern recognition by opcode mix. Use when mapping ASM to source code, finding which source line a GPU instruction comes from, or doing DWARF line table analysis on GPU binaries.
Its SKILL.md is about 1.8k tokens, which your agent loads only when the skill is triggered. It is a single SKILL.md file with no bundled scripts.
It sits in Development. It works with C++. The licence is Apache-2.0.
5 steps, taken from the first numbered list in SKILL.md.
Read from SKILL.md and the folder at commit 0187b3b. It shows what the files ask for, not the result of running them.
Pre-approves nothing: there is no allowed-tools line, so your agent's usual permission prompts apply.
From allowed-tools in the SKILL.md frontmatter.
No scripts in the folder and no shell commands in SKILL.md (its code samples are bash, asm, json and cpp).
From the folder's file list and the shell code blocks in SKILL.md.
No URLs in SKILL.md.
From URLs in SKILL.md, links to its own repository left out.
Names no API keys, tokens, secrets or passwords.
From names ending in _API_KEY, _TOKEN, _SECRET, _KEY or _PASSWORD in SKILL.md.
Asm Source Mapping loads about 1.8k tokens when it runs. Until then it costs about 97 tokens; SKILL.md has 540 words of instructions outside code blocks.
Estimates: characters ÷ 4, the usual rule of thumb; real counts depend on the model's tokenizer. Scripts and assets cost tokens only if the agent reads them.
The automated check found no risky patterns in SKILL.md.
Automated static check — not a guarantee. Review scripts before installing. It scans the text of SKILL.md for risky patterns (piping downloads into a shell, reading credential files, hidden Unicode, destructive commands); files beside SKILL.md are not scanned.
The full file from intel/torch-xpu-ops at commit 0187b3b, republished under its Apache-2.0 licence (© intel). 540 words, ~1,787 tokens.
.claude/skills/asm-source-mapping/SKILL.md (or your agent's skills folder).Given one or more GPU instruction addresses (IPs) and an ASM directory, resolve each IP to the original SYCL/DPC++ source file:line and an optional structural construct label.
Debug line info flows: C++ source → LLVM IR !dbg → SPIR-V OpLine → IGC vISA DebugLoc → zebin .debug_line (byte offset → file:line). The zebin's
.debug_line section maps GPU ISA offsets to source, and is the primary data
source for this skill. Requires -g at compile time (-gline-tables-only is
not supported for spir64_gen AOT targets and is silently ignored by the
compiler now and may fix it in the future version). oneDNN ngen has no .debug_line — pattern recognition is the only
option.
Step 1: Try zebin .debug_line → method = "debug-line"
↓ (not found)
Step 2: Try IGC ASM inline comments → method = "debug-line"
↓ (not found)
Step 3: Fallback: pattern recognition → method = "pattern-recognition"Rule: if Method 1 succeeds, STOP. Do NOT run pattern-recognition for line attribution. Only add an optional construct label as enrichment.
.debug_line (primary)The zebin ELF (e.g. dumped_zebin_module_0.elf) must contain a .debug_line
section (present when
compiled with -g or -gline-tables-only).
Locate the zebin ELF.
The zebin is in the extraction working directory (where
DumpZEBin=1 NEOReadDebugKeys=1 dumped the .elf file). Typical locations:
<workdir>/dumped_zebin_module_N.elf (from DumpZEBin runtime dump)ZEBIN=<path to zebin ELF from extraction step>
file "$ZEBIN" # must show: ELF 64-bit LSB relocatable, *unknown arch 0xcd*Verify .debug_line exists.
readelf -S "$ZEBIN" | grep -q .debug_line || { echo "NO debug_line"; exit 1; }Decode the line table.
readelf --debug-dump=decodedline "$ZEBIN" 2>/dev/nullNote: readelf may warn about unknown reloc types for e_machine=205
(Intel GPU). Ignore it — the decoded output is still correct.
Output format:
shift_reduce.cpp 77 0 x
shift_reduce.cpp 91 0xe8 x
shift_reduce.cpp 93 0x238 xBuild a lookup table. Parse into sorted (offset, file, line) tuples.
For a given IP offset, find the entry with the largest offset ≤ IP
(floor lookup).
Correlate with ASM file. The .asm file uses labels L<N> where N
is the decimal byte offset in .text. Map IP offset → label → ASM line.
JIT-compiled kernels (IGC_ShaderDumpEnable=1) produce .asm files with
inline source comments. Two formats exist depending on IGC version:
// Format A (newer IGC, requires -g):
// Line 8: int val = buf[it.get_global_id(0)];
(W) send.ugm (32|M0) r16 r14 ...
// Line 9: val += sycl::shift_group_left(...);
(W) mov (16|M0) r20.0<1>:d r16.0<1;1,0>:d
// Format B (older IGC):
(W) add (M1, 16) r14.0<1>:d r14.0<1;1,0>:d 0x40:w // shift_reduce.cpp:93Scan for // Line <N>: or // <file>:<line> patterns. For each IP,
walk backward to the nearest preceding annotated line.
Use ONLY when Methods 1 and 2 both fail (no -g at compile time, or
oneDNN ngen JIT which has no DWARF).
Classify each IP by surrounding instruction mix:
| Instruction pattern | Source construct |
|---|---|
Dense dpas chain, 8×repeat | GEMM tile (matmul accumulate) |
mul :f + broadcast src1 | Scalar rescale (softmax denominator) |
exp2 :f / log2 :f | Softmax numerics |
mov :bf :f + store | BF16 epilogue / output write |
send.slm | SLM load/store |
send.slm store + fence.slm + send.slm load + add :f | Reduction tree (workgroup reduce) |
send.ugm | Global memory load/store |
send.gtwy + sync.bar | Barrier / synchronization |
VxH indirect mov r[a0.0] | Sub-group shuffle (shift_group_left) |
JSON array, one entry per IP:
[
{
"ip": "0x238",
"asm_offset": 568,
"asm_label": "L568",
"sycl_file": "shift_reduce.cpp",
"sycl_line": 93,
"source_construct": "ALU compute (value += other)",
"method": "debug-line"
}
]Note: asm_label = L<decimal byte offset> (matching .asm file labels).
asm_offset is the same value as an integer. ip is the hex form.
.debug_line (Method 1)Source (compiled with icpx -fsycl -g -fsycl-targets=spir64_gen):
class ShiftReduceKernel;
q.parallel_for<ShiftReduceKernel>(nd_range<1>(1024, 32), [=](nd_item<1> it) {
int val = buf[it.get_global_id(0)]; // line 8
val += sycl::shift_group_left(it.get_sub_group(), val, 1); // line 9
buf[it.get_global_id(0)] = val; // line 10
});$ readelf --debug-dump=decodedline dumped_zebin_module_0.elf
# Output (offsets and lines vary by compiler version):
# <source_file> <line> <offset>
# shift_reduce.cpp 8 0xe8
# shift_reduce.cpp 9 0x238
# ...
# Given IP: 0x238
# Floor lookup: largest offset ≤ 0x238 → line 9
# Result:
# ip=0x238 → shift_reduce.cpp:9
# construct="ALU compute (shift_group_left)"
# method=debug-line# From IGC_ShaderDumpEnable=1 dump (compiled with -g):
// Line 8: int val = buf[it.get_global_id(0)];
(W) send.ugm (32|M0) r16 r14 ...
// Line 9: val += sycl::shift_group_left(...);
(W) mov (16|M0) r20.0<1>:d r16.0<1;1,0>:d
# IP is at the mov instruction → walk backward → "// Line 9"
# Result:
# sycl_line=9 method=debug-line# No .debug_line, no // comments (oneDNN ngen JIT kernel)
# IP surrounded by dense dpas instructions:
# dpas (8) r40.0<1>:f r36.0<8;8,1>:hf r20.0<8;4,1>:hf
# dpas (8) r48.0<1>:f r44.0<8;8,1>:hf r20.0<8;4,1>:hf
# Result:
# sycl_file=null construct="GEMM tile/dpas" method=pattern-recognition© intel, Apache-2.0. Rendered from Markdown: HTML in the file is shown as text, images as links, and headings moved down two levels. Raw file
Just SKILL.md in .claude/skills/asm-source-mapping of intel/torch-xpu-ops.
Open the folder on GitHubat commit 0187b3b
Asm Source Mapping next to the 5 skills that share the most tags, products or categories with it. Stars are the repository's; “used in” counts other GitHub owners with a copy.
| Skill | Stars | Used in | Tokens | Auto-check | Licence | Repo updated |
|---|---|---|---|---|---|---|
| Asm Source Mapping this skillintel/torch-xpu-ops | 115 | — | ~1.8k | Automated safety check: Pass | Apache-2.0 | |
| Qt C++ Code Reviewx-tools-author/x-tools | 1.1k | 2 repos | ~4.3k | Automated safety check: Pass | BSD-3-Clause | |
| YugabyteDB ASH Instrumentationyugabyte/yugabyte-db | 11k | — | ~4.5k | Automated safety check: Pass | Custom licence | |
| pybind11 Release Preparationpybind/pybind11 | 18k | — | ~1.7k | Automated safety check: Pass | Custom licence | |
| Qt Cpp ReviewSerial-Studio/Serial-Studio | 7.2k | — | ~4.3k | Automated safety check: Pass | Custom licence | |
| Paddle Eager GraphPaddlePaddle/Paddle | 24k | — | ~562 | Automated safety check: Pass | Apache-2.0 |
x-tools-author/x-tools
Read-only review of Qt6 C++ code that combines a deterministic lint script with six parallel analysis agents and reports only high-confidence issues.
yugabyte/yugabyte-db
Procedure for adding or changing YugabyteDB Active Session History wait states in TServer and DocDB C++ code, including the macro to use for sync and async paths.
pybind/pybind11
Opens the pybind11 release-preparation pull request: picking the release base, bumping the version in common.h and integrating the changelog, following docs/release.rst.
Serial-Studio/Serial-Studio
Qt6/C++ deep code review for Serial Studio. An agent skill from Serial-Studio/Serial-Studio.
PaddlePaddle/Paddle
A skill your agent uses when navigating Paddle eager-mode (dynamic graph) source code, tracing forward/backward execution, debugging autograd issues, understanding PyLayer, or investigating…
microsoft/onnxruntime
Builds ONNX Runtime from source with its build scripts, explaining the update, build and test phases, key flags and where the build output lands.
intel/torch-xpu-ops
Select the Intel GPU device to use when a system has multiple Intel GPU devices.
intel/torch-xpu-ops
Check PyTorch ciflow/xpu (xpu.yml) on the main branch, collect the failing XPU test cases from the most recent completed run(s), analyze the ROOT CAUSE of each failure with AI, and produce a list…
intel/torch-xpu-ops
Convert PyTorch ATDISPATCH macros to ATDISPATCHV2 format in ATen C++ code.
intel/torch-xpu-ops
Review pull requests for XPU operator or backend code. An agent skill from intel/torch-xpu-ops.
intel/torch-xpu-ops
Guide users through creating Agent Skills for Claude Code. An agent skill from intel/torch-xpu-ops.
intel/torch-xpu-ops
Read the evidence a nightly UT run produced, decide which failures share a root cause and which are machine breakage rather than product bugs, and write one issue draft per root cause to drafts.json.
Works with
Categories
Map Intel GPU ISA instruction addresses to precise SYCL/DPC++ source file:line numbers. Asm Source Mapping is an agent skill from intel/torch-xpu-ops, published by the product's own GitHub organization. Map Intel GPU ISA instruction addresses to precise SYCL/DPC++ source file:line numbers.
Asm Source Mapping fits situations like: mapping ASM to source code; finding which source line a GPU instruction comes from; doing DWARF line table analysis on GPU binaries.
Run `npx skills add intel/torch-xpu-ops --skill asm-source-mapping -a claude-code`. Or copy the skill folder (.claude/skills/asm-source-mapping in intel/torch-xpu-ops) into .claude/skills/asm-source-mapping in your project. Claude Code loads it when a task matches its description.
Run `npx skills add intel/torch-xpu-ops --skill asm-source-mapping -a codex`. Or copy the skill folder (.claude/skills/asm-source-mapping in intel/torch-xpu-ops) into .agents/skills/asm-source-mapping in your project. Codex loads it when a task matches its description.
Cursor, Gemini CLI, GitHub Copilot and OpenCode also load SKILL.md folders. With the skills CLI, run `npx skills add intel/torch-xpu-ops --skill asm-source-mapping -a cursor` (or -a gemini-cli, github-copilot or opencode for the others). To copy it by hand, put the folder in .cursor/skills/asm-source-mapping, .gemini/skills/asm-source-mapping, .github/skills/asm-source-mapping and .opencode/skills/asm-source-mapping in your project.
SKILL.md names no scripts, command-line tools or credentials: Asm Source Mapping is instructions for the agent only.
SKILL.md contains no URLs. Any network use would come from the scripts or tools the agent runs. This is read from the text; nothing was executed.
Our automated static check of SKILL.md found no risky patterns, such as piping downloads into a shell, reading credential files or hidden Unicode. It is not a guarantee. Review the folder before installing.
Asm Source Mapping is published under the Apache-2.0 licence (the repository's licence). It allows redistribution, so the full SKILL.md is shown on this page.
About 1.8k tokens (SKILL.md is roughly 7.1k characters). Agents keep only the skill's name and description in context until a task matches; then they load SKILL.md in full.
Skills that share tags, products or a category with Asm Source Mapping: Qt C++ Code Review (x-tools-author/x-tools, 1.1k stars), YugabyteDB ASH Instrumentation (yugabyte/yugabyte-db, 11k stars), pybind11 Release Preparation (pybind/pybind11, 18k stars) and Qt Cpp Review (Serial-Studio/Serial-Studio, 7.2k stars). The comparison table on this page puts their stars, adoption, token cost, safety result and licence side by side.
intel (a GitHub organization, an official publisher) maintains it in intel/torch-xpu-ops, which has 115 GitHub stars. The repository holds 29 skills in this directory. The repository was last updated on October 6, 2026.
Source: intel/torch-xpu-ops on GitHub. Facts on this page come from the repository at the commit we read; the author's words are quoted as theirs.